@@ -4,9 +4,7 @@
 **
 **  MODULE
 **
-**      $RCSfile: version.h,v $
-**      $Revision: 3.71 $
-**      $Date: 1996/08/18 20:37:06 $
+**      file: version.h
 **
 **  DESCRIPTION
 **
@@ -91,6 +89,8 @@
 */
 #ifndef __VERSION_H__
 # define __VERSION_H__              1
-# define VERSION                    "3.71 (04 mar 2019)"
+# define PACKAGE_DATE               "(22 may 2022)"
+# define EXT_VERSION                PACKAGE_VERSION
+# define VERSION                    PACKAGE_VERSION " " PACKAGE_DATE
 #endif                          /* __VERSION_H__ */
 
