@@ -191,6 +191,7 @@ struct DIMENSION_REC {
 #define FNT1            235
 #define XXX1            239
 #define FNT_DEF_1       243
+#define FNT_DEF_4       246
 #define PRE             247
 #define POST            248
 #define POST_POST       249
@@ -1809,6 +1810,56 @@ void out_string(FILE *in, FILE *out, int len)
     }
 }
 
+
+void fontdef(FILE *dvi, int nn)
+{
+    int code, tmp = 0;
+    uint csum;
+
+    fprintf(fp_out, " %d", read_n(dvi, nn));    /* code */
+    csum = read_n(dvi, 4);
+    if(csum)
+        fprintf(fp_out,
+            (f_dtl&DTL_FNTDEF)?((f_dtl&DTL_OCT)?" %o%s":" 0%o%s"):
+            " 0x%X%s", csum, MSG("/c-sum"));            /* chksum */
+    else
+        fprintf(fp_out, " 0%s", MSG("/c-sum"));
+    fprintf(fp_out, " %u%s", read_long(dvi), MSG("/s-size"));   /* scaled size */
+    fprintf(fp_out, " %u%s", read_long(dvi), MSG("/d-size"));   /* design size */
+    tmp = (uchar)read_byte(dvi);
+    fprintf(fp_out, " %d%s", tmp, MSG("/dir"));                 /* len:directry */
+    code = (uchar)read_byte(dvi);
+    fprintf(fp_out, " %d%s '", code, MSG("/name"));             /* len:name */
+    while (tmp-- > 0)
+        putc(read_byte(dvi), fp_out);
+    if((f_dtl&DTL_FNTNAME))
+        fputs("' '", fp_out);
+    while(code-- > 0)
+        putc(read_byte(dvi), fp_out);
+    fputs("'\n", fp_out);
+}
+
+
+uchar skipnop(FILE *dvi)
+{
+    uchar code;
+
+    while (code = (uchar)getc(dvi)) {
+        if (code >= FNT_DEF_1 && code <= FNT_DEF_4) {
+            fprintf(fp_out, "%s", (f_dtl&DTL_CMD)?c235_name[code-FNT1]:cmd235_name[code-FNT1]);
+            fontdef(dvi, code-FNT_DEF_1+1);
+            continue;
+        }
+        else if (code == NOP) {
+            fprintf(fp_out, "%s\n", (f_dtl&DTL_CMD)?c128_name[code-128]:cmd128_name[code-128]);
+            continue;
+        }
+        break;
+    }
+    return code;
+}
+
+
 /* preamble */
 void transpre(FILE *dvi)
 {
@@ -1826,6 +1877,7 @@ void transpre(FILE *dvi)
     putc('\'', fp_out);
     out_string(dvi, fp_out, len);
     fputs("\'\n", fp_out);
+    skipnop(dvi);
 }
 
 
@@ -1981,27 +2033,7 @@ skip_m:             while(mode-- > 0)
                         continue;
 
                     case (2):   /* fntdef */
-                        fprintf(fp_out, " %d", read_n(dvi, mode & 0xf));    /* code */
-                        csum = read_n(dvi, 4);
-                        if(csum)
-                            fprintf(fp_out, 
-                                (f_dtl&DTL_FNTDEF)?((f_dtl&DTL_OCT)?" %o%s":" 0%o%s"):
-                                " 0x%X%s", csum, MSG("/c-sum"));            /* chksum */
-                        else
-                            fprintf(fp_out, " 0%s", MSG("/c-sum")); 
-                        fprintf(fp_out, " %u%s", read_long(dvi), MSG("/s-size"));   /* scaled size */
-                        fprintf(fp_out, " %u%s", read_long(dvi), MSG("/d-size"));   /* design size */
-                        tmp = (uchar)read_byte(dvi);
-                        fprintf(fp_out, " %d%s", tmp, MSG("/dir"));                 /* len:directry */
-                        code = (uchar)read_byte(dvi);
-                        fprintf(fp_out, " %d%s '", code, MSG("/name"));             /* len:name */
-                        while (tmp-- > 0)
-                            putc(read_byte(dvi), fp_out);
-                        if((f_dtl&DTL_FNTNAME))
-                            fputs("' '", fp_out);
-                        while(code-- > 0)
-                            putc(read_byte(dvi), fp_out);
-                        fputs("'\n", fp_out);
+                        fontdef(dvi, mode & 0xf);
                         continue;
 /*
                   case (3):
@@ -2027,6 +2059,7 @@ skip_m:             while(mode-- > 0)
     if(f_pos)
         fprintf(fp_out, "%ld: ", ftell(dvi)-1);
     fputs((code==EOP)?"eop\n":"post_post", fp_out);
+    if (code==EOP) skipnop(dvi);
     return ftell(dvi);
 }
 
@@ -2339,7 +2372,7 @@ er1:        fprintf(stderr, "Command after POST_POST\n");
             goto err;
         }
         if(!f_in){
-er2:        if(code < FNT_DEF_1 && code != BOP){
+er2:        if(code < FNT_DEF_1 && code != NOP && code != BOP){
                 fprintf(stderr, "This command shoud be after BOP\n");
                 goto err;
             }
@@ -2639,6 +2672,10 @@ put_num:        write_n(a2i(get_next(base)), sub_number);
                 putc(a2i(get_next(s)), fp_out);
                 break;
 
+            case NOP:
+                putc(code, fp_out);
+                break;
+
             default:
 err:            fprintf(stderr, "Error(line %d, code %d): %s\n", line, code, tmp_buf);
                 Exit(1);
