@@ -1,5 +1,5 @@
 /* This is extractbb, a bounding box extraction program.
-    Copyright (C) 2008-2020 by Jin-Hwan Cho and Matthias Franz
+    Copyright (C) 2008-2023 by Jin-Hwan Cho and Matthias Franz
     and the dvipdfmx project team.
 
     This program is free software; you can redistribute it and/or modify
@@ -52,7 +52,7 @@ static int Include_Page = 1;
 static void show_version(void)
 {
   fprintf (stdout, "\nThis is %s Version " VERSION "\n", my_name);
-  fprintf (stdout, "\nCopyright (C) 2008-2020 by Jin-Hwan Cho and Matthias Franz\n");
+  fprintf (stdout, "\nCopyright (C) 2008-2023 by Jin-Hwan Cho and Matthias Franz\n");
   fprintf (stdout, "\nThis is free software; you can redistribute it and/or modify\n");
   fprintf (stdout, "it under the terms of the GNU General Public License as published by\n");
   fprintf (stdout, "the Free Software Foundation; either version 2 of the License, or\n");
@@ -304,7 +304,7 @@ int extractbb (int argc, char *argv[])
 
   pdf_files_init();
 
-  pdf_set_version(PDF_VERSION_MAX);
+  dpx_conf.is_xbb = 1;
 
   opterr = 0;
   
