@@ -5,7 +5,7 @@
 #   Copyright (C) 2004  F J Tilmann (tilmann@gfz-potsdam.de)
 #
 # Repository:         https://github.com/ftilmann/latexdiff
-# CTAN page:          http://www.ctan.org/tex-archive/support/latexdiff
+# CTAN page:          http://www.ctan.org/pkg/latexdiff
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -21,6 +21,10 @@
 #
 # Detailed usage information at the end of the file
 # Note: version number now keeping up with latexdiff
+# Version 1.3.2 
+#   - Functionality -c option: if comment to be deleted is the last in the line replace new line by space character. This is done to avoid errors or unintended line breaks from empty lines after deletion (which are implicit \par)
+#   - Introduce option -r / --replace which removes markup of the form \replaced{new text}{old text} in the changes package
+# Version 1.3.1  (Bug fix) Remove some deprecation warnings due to uncommented left parentheses 
 # Version 1.0.2  Option --version
 # Version 1.0.1   no changes to latexrevise
 # Version 0.3   Updated for compatibility with latexdiff 0.3 output (DIFAUXCMD removal)
@@ -31,8 +35,8 @@ use strict;
 use warnings;
 
 my $versionstring=<<EOF ;
-This is LATEXREVISE 1.0.2
-  (c) 2005-2012 F J Tilmann
+This is LATEXREVISE 1.3.2
+  (c) 2005-2021 F J Tilmann
 EOF
 
 # Markup strings (make sure that this are set to the same value as in 
@@ -62,20 +66,36 @@ my $PREAMBLEXTBEG='^%DIF PREAMBLE EXTENSION ADDED BY LATEXDIFF$';
 my $PREAMBLEXTEND='^%DIF END PREAMBLE EXTENSION ADDED BY LATEXDIFF$';
 
 
-my $pat0 = '(?:[^{}]|\\\{|\\\})*';
-my $pat1 = '(?:[^{}]|\\\{|\\\}|\{'.$pat0.'\})*';
-my $pat2 = '(?:[^{}]|\\\{|\\\}|\{'.$pat1.'\})*';
-my $pat3 = '(?:[^{}]|\\\{|\\\}|\{'.$pat2.'\})*';
-my $pat4 = '(?:[^{}]|\\\{|\\\}|\{'.$pat3.'\})*';
+#my $pat0 = '(?:[^{}]|\\\{|\\\})*';
+#my $pat1 = '(?:[^{}]|\\\{|\\\}|\{'.$pat0.'\})*';
+#my $pat2 = '(?:[^{}]|\\\{|\\\}|\{'.$pat1.'\})*';
+#my $pat3 = '(?:[^{}]|\\\{|\\\}|\{'.$pat2.'\})*';
+#my $pat4 = '(?:[^{}]|\\\{|\\\}|\{'.$pat3.'\})*';
+
+my $pat0 = '(?:[^{}])*';
+my $pat_n = $pat0;
+# if you get "undefined control sequence MATHBLOCKmath" error, increase the maximum value in this loop
+for (my $i_pat = 0; $i_pat < 20; ++$i_pat){
+  $pat_n = '(?:[^{}]|\{'.$pat_n.'\}|\\\\\{|\\\\\})*';
+  # Actually within the text body, quoted braces are replaced in pre-processing. The only place where 
+  # the last part of the pattern matters is when processing the arguments of context2cmds in the preamble
+  # and these contain a \{ or \} combination, probably rare.
+  # It should thus be fine to use the simpler version below.
+  ###  $pat_n = '(?:[^{}]|\{'.$pat_n.'\})*';
+}
+
+
 my $brat0 = '(?:[^\[\]]|\\\[|\\\])*'; 
 
 
+
+
 my ($input,$preamble,$body,$post);
 my (@matches);
 my ($cnt,$prematch,$postmatch);
 my ($help,$version);
 my ($verbose,$quiet,$accept,$decline,$simplify)=(0,0,0,0,0);
-my ($comment,$comenv,$markup,$markenv);
+my ($comment,$comenv,$markup,$markenv,$replace);
 
 # A word unlikely ever to be used in a real latex file
 my $someword='gobbledegooksygook';
@@ -88,6 +108,7 @@ GetOptions('accept|a' => \$accept,
 	   'comment-environment|e=s' => \$comenv,
 	   'markup|m=s' => \$markup,
 	   'markup-environment|n=s' => \$markenv,
+	   'replace|r=s' => \$replace,
 	   'no-warnings|q' => \$verbose,
            'version' => \$version,
 	   'verbose|V' => \$verbose,
@@ -98,12 +119,13 @@ if ( $help ) {
 }
 
 if ( $version ) {
-  die $versionstring ; 
+  print STDERR $versionstring ; 
+  exit 0;
 }
 
 
 if ( ($accept &&  $decline) || ($accept && $simplify) || ($decline && $simplify) ) {
-  die '-a,-d and -s options are mutually axclusive. Type latexrevise -h to get more help.';
+  die '-a,-d and -s options are mutually exclusive. Type latexrevise -h to get more help.';
 }
 
 
@@ -178,7 +200,7 @@ elsif ( $decline) {
 
   # Mop up tokens. This must be done already now as otherwise 
   # detection of white-space problems does not work
-  $cnt = $body =~ s/${DELOPEN}($pat4)${DELCLOSE}/$1/sg;
+  $cnt = $body =~ s/${DELOPEN}($pat_n)${DELCLOSE}/$1/sg;
   # remove markup of deleted commands 
   $cnt +=   $body =~ s/${DELCMDOPEN}(.*?)${DELCMDCLOSE}/$1/sg ;
   $cnt +=   $body =~ s/${DELCMDOPEN}//g ;
@@ -217,13 +239,13 @@ elsif ( $decline) {
 # remove any remaining tokens
 if ( $accept || $decline || $simplify ) {
   # first substitution command deals with special case of added paragraph
-  $cnt = $body =~ s/${ADDOPEN}($pat4)\n${ADDCLOSE}\n/$1\n/sg;
-  $cnt += $body =~ s/${ADDOPEN}($pat4)${ADDCLOSE}/$1/sg;
+  $cnt = $body =~ s/${ADDOPEN}($pat_n)\n${ADDCLOSE}\n/$1\n/sg;
+  $cnt += $body =~ s/${ADDOPEN}($pat_n)${ADDCLOSE}/$1/sg;
   $cnt==0 || warn 'Remaining $ADDOPEN tokens in DECLINE mode\n' unless ( $quiet || $accept || $simplify );
 }
 if ($accept || $simplify ) {
   # Note: in decline mode these commands have already been removed above
-  $cnt = $body =~ s/${DELOPEN}($pat4)${DELCLOSE}/$1/sg;
+  $cnt = $body =~ s/${DELOPEN}($pat_n)${DELCLOSE}/$1/sg;
   #### remove markup of deleted commands 
   $cnt +=   $body =~ s/${DELCMDOPEN}(.*?)${DELCMDCLOSE}/$1/sg ;
   $cnt +=   $body =~ s/${DELCMDOPEN}//g ;
@@ -238,11 +260,11 @@ if ($accept || $simplify ) {
 
 # Remove comment commands
 if (defined($comment)) {
-  print STDERR "Removing \\$comment\{..\} sequences ..." if $verbose;
+  print STDERR "Removing \\$comment\{..\} sequences (incl. argument)..." if $verbose;
   # protect $comments in comments by making them look different
   $body =~ s/(%.*)${comment}(.*)$/$1${someword}$2/mg ;
   # carry out the substitution
-  $cnt = 0 + $body =~ s/\\${comment}(?:\[${brat0}\])?\{${pat4}\}//sg ;
+  $cnt = 0 + $body =~ s/\\${comment}(?:\[${brat0}\])?\{${pat_n}\}(?: *\n)?//sg ;
   print STDERR "$cnt matches found and removed.\n" if $verbose;
   # and undo the protection substitution
   $body =~ s/(%.*)${someword}(.*)$/$1${comment}$2/mg ;
@@ -257,11 +279,11 @@ if (defined($comenv)) {
 }
 
 if (defined($markup)) {
-  print STDERR "Removing \\$markup\{..\} cpmmands ..." if $verbose;
+  print STDERR "Removing \\$markup\{..\} commands (leaving argument)..." if $verbose;
   # protect $markups in comments by making them look different
   $body =~ s/(%.*)${markup}(.*)$/$1${someword}$2/mg ;
   # carry out the substitution
-  $cnt = 0 + $body =~ s/\\${markup}(?:\[${brat0}\])?\{(${pat4})\}/$1/sg ;
+  $cnt = 0 + $body =~ s/\\${markup}(?:\[${brat0}\])?\{(${pat_n})\}/$1/sg ;
   print STDERR "$cnt matches found and removed.\n" if $verbose;
   # and undo the protection substitution
   $body =~ s/(%.*)${someword}(.*)$/$1${markup}$2/mg ;
@@ -275,6 +297,16 @@ if (defined($markenv)) {
   $body =~ s/(%.*)${someword}/$1${markenv}/mg ;
 }
 
+if (defined($replace)) {
+  print STDERR "Removing \\$replace\{..\}\{..\} commands (leaving 1st and discarding 2nd argument))..." if $verbose;
+  # protect $markups in comments by making them look different
+  $body =~ s/(%.*)${replace}(.*)$/$1${someword}$2/mg ;
+  # carry out the substitution
+  $cnt = 0 + $body =~ s/\\${replace}(?:\[${brat0}\])?\{(${pat_n})\}\s?\{(${pat_n})\}/$1/sg ;
+  print STDERR "$cnt matches found and removed.\n" if $verbose;
+  # and undo the protection substitution
+  $body =~ s/(%.*)${someword}(.*)$/$1${replace}$2/mg ;
+}
 
 if ( length $preamble ) {
   print "$preamble\\begin{document}${body}\\end{document}$post";
@@ -332,7 +364,7 @@ sub splitdoc {
 
 
 sub usage {
-  die <<"EOF"; 
+  print STDERR <<"EOF"; 
 Usage: $0 [OPTIONS] [diff.tex] > revised.tex
 
 Read a file diff.tex (output of latexdiff), and remove its markup. 
@@ -373,12 +405,12 @@ In SIMPLIFY mode the preamble is left unmodified.
 
 Note that the three mode options are mutually exclusive. If no mode option is given,
 latexrevise simply removes user annotations and markup according to the following four
-options.
+options; these functions can be very useful outside the latexdiff context, too.
 
 
 -c cmd
 --comment=cmd     Remove \\cmd{...}.  cmd is supposed to mark some explicit 
-                  anotations which should be removed from the file before 
+                  annotations which should be removed from the file before 
                   release.
 
 -e envir
@@ -397,6 +429,11 @@ options.
 --markup-environment=envir  
                   Similarly, remove \\begin{envir} and \\end{envir} commands,
                   but leave content of the environment in the text.
+
+
+-r cmd
+--replace=cmd    For constructions \\cmd{..}{..}, remove the command, leave the 
+                 content of first argument, and delete second argument. 
                   
 -q
 --no-warnings     Do not warn users about \\DIDadd{..} or \\DIFdel statements
@@ -406,6 +443,7 @@ options.
 --verbose         Verbose output
 
 EOF
+  exit 0;
 }
 
 =head1 NAME
@@ -463,14 +501,14 @@ markup from body).
 
 Note that the three mode options are mutually exclusive.  If no mode option is given,
 I<latexrevise> simply removes user annotations and markup according to the following four
-options.
+options. These functions can be very useful outside the latexdiff context, too
 
 =over 4
 
 =item B<-c cmd> or B<--comment=cmd>
 
 Remove C<\cmd{...}> sequences.  C<cmd> is supposed to mark some explicit 
-anotations which should be removed from the file before 
+annotations which should be removed from the file before 
 release.
 
 =item B<-e envir> or B<--comment-environment=envir> 
@@ -492,6 +530,13 @@ turn C<\cmd{abc}> into C<abc>.
 
 Similarly, remove C<\begin{envir}> and C<\end{envir}> commands but 
 leave content of the environment in the text.
+
+=item B<-r cmd> or B<--replace=cmd>
+
+For constructions C<\\cmd{..}{..}>, remove the command, leave the 
+content of first argument, and delete second argument, i.e. turn 
+C<\cmd{abc}{def}> into C<abc>.
+
                   
 
 =item B<-V> or B<--verbose>
@@ -508,8 +553,9 @@ which should have been removed already.
 =head1 BUGS
 
 The current version is a beta version which has not yet been
-extensively tested, but worked fine locally.  Please submit bug reports using the issue tracker of the github repository page I<https://github.com/ftilmann/latexdiff.git>, 
-or send them to I<tilmann -- AT -- gfz-potsdam.de>..  Include the serial number of I<latexrevise>
+extensively tested. It has not been actively maintained so might not process output of newer versions of latexdiff entirely correctly.
+Please submit bug reports using the issue tracker of the github repository page I<https://github.com/ftilmann/latexdiff.git>, 
+or send them to I<tilmann -- AT -- gfz-potsdam.de>.  Include the serial number of I<latexrevise>
 (Option --version).  If you come across latexdiff
 output which is not processed correctly by I<latexrevise> please include the
 problem file as well as the old and new files on which it is based,
