@@ -1,23 +1,23 @@
-/*373:*/
-#line 7563 "hint.w"
+/*383:*/
+#line 7641 "hint.w"
 
 #ifndef _HRENDER_H
 #define _HRENDER_H
-/*276:*/
-#line 5047 "hint.w"
+/*284:*/
+#line 5094 "hint.w"
 
 #define MARK_BIT 0x1
 #define FOCUS_BIT 0x2
-/*:276*//*295:*/
-#line 5497 "hint.w"
+/*:284*//*302:*/
+#line 5485 "hint.w"
 
 #define LINK_BIT 0x4
-/*:295*//*320:*/
-#line 6124 "hint.w"
+/*:302*//*329:*/
+#line 6154 "hint.w"
 
 #define SP2PT(X) ((X)/(double)(1<<16))
-/*:320*/
-#line 7566 "hint.w"
+/*:329*/
+#line 7644 "hint.w"
 
 
 extern int page_h,page_v;
@@ -40,6 +40,12 @@ extern bool hint_prev_mark(void);
 extern bool hint_next_mark(void);
 extern double hint_get_fpos(void);
 extern uint64_t hint_set_fpos(double fpos);
-
+extern void hint_render_on(void);
+extern void hint_render_off(void);
+extern void hint_dark(int dark);
+extern void hint_gamma(double gamma);
+extern int hint_print_on(int w,int h,int bpr,int bpp,unsigned char*bits);
+extern int hint_print_off(void);
+extern int hint_print(unsigned char*bits);
 #endif
-/*:373*/
+/*:383*/
