@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.0.2.  */
+/* A Bison parser, made by GNU Bison 3.0.4.  */
 
 /* Bison implementation for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "3.0.2"
+#define YYBISON_VERSION "3.0.4"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -115,7 +115,7 @@ extern int yydebug;
     STRING = 260,
     LEFTARROW = 261,
     RIGHTARROW = 262,
-    INPUT = 263,
+    MYINPUT = 263,
     OUTPUT = 264,
     ALIASES = 265,
     STATES = 266,
@@ -135,7 +135,7 @@ extern int yydebug;
 #define STRING 260
 #define LEFTARROW 261
 #define RIGHTARROW 262
-#define INPUT 263
+#define MYINPUT 263
 #define OUTPUT 264
 #define ALIASES 265
 #define STATES 266
@@ -481,7 +481,7 @@ static const yytype_uint16 yyrline[] =
 static const char *const yytname[] =
 {
   "$end", "error", "$undefined", "NUMBER", "ID", "STRING", "LEFTARROW",
-  "RIGHTARROW", "INPUT", "OUTPUT", "ALIASES", "STATES", "TABLES",
+  "RIGHTARROW", "MYINPUT", "OUTPUT", "ALIASES", "STATES", "TABLES",
   "EXPRESSIONS", "PUSH", "POP", "DIV", "MOD", "BEG", "END", "'+'", "'-'",
   "'*'", "';'", "'['", "']'", "'='", "'{'", "'}'", "','", "'<'", "'>'",
   "'.'", "'^'", "'('", "')'", "'|'", "'\\\\'", "'$'", "'#'", "$accept",
